@@ -1,3 +1,4 @@
+//go:build circuitplay_bluefruit
 // +build circuitplay_bluefruit
 
 package machine
@@ -38,6 +39,7 @@ const (
 const (
 	LED       = D13
 	NEOPIXELS = D8
+	WS2812    = D8
 
 	BUTTONA = D4
 	BUTTONB = D5
@@ -56,18 +58,13 @@ const (
 	UART_RX_PIN = P0_30 // PORTB
 )
 
-// UART0 is the USB device
-var (
-	UART0 = USB
-)
-
 // I2C pins
 const (
 	SDA_PIN = P0_05 // I2C0 external
 	SCL_PIN = P0_04 // I2C0 external
 
-	SDA1_PIN = P0_00 // I2C1 internal
-	SCL1_PIN = P0_01 // I2C1 internal
+	SDA1_PIN = P1_10 // I2C1 internal
+	SCL1_PIN = P1_12 // I2C1 internal
 )
 
 // SPI pins (internal flash)
@@ -87,3 +84,7 @@ var (
 	usb_VID uint16 = 0x239A
 	usb_PID uint16 = 0x8045
 )
+
+var (
+	DefaultUART = UART0
+)
