@@ -1,3 +1,4 @@
+//go:build linux
 // +build linux
 
 package runtime
@@ -11,3 +12,9 @@ const (
 	flag_MAP_PRIVATE   = 0x2
 	flag_MAP_ANONYMOUS = 0x20
 )
+
+// Source: https://github.com/torvalds/linux/blob/master/include/uapi/linux/time.h
+const (
+	clock_REALTIME      = 0
+	clock_MONOTONIC_RAW = 4
+)
