@@ -1,3 +1,4 @@
+//go:build esp8266
 // +build esp8266
 
 package runtime
@@ -23,8 +24,6 @@ func putchar(c byte) {
 //export rom_i2c_writeReg
 func rom_i2c_writeReg(block, host_id, reg_add, data uint8)
 
-func postinit() {}
-
 //export main
 func main() {
 	// Clear .bss section. .data has already been loaded by the ROM bootloader.
