@@ -1,3 +1,4 @@
+//go:build nintendoswitch
 // +build nintendoswitch
 
 package runtime
@@ -43,8 +44,6 @@ var (
 	totalHeap = uint64(0)
 )
 
-func postinit() {}
-
 func preinit() {
 	// Unsafe to use heap here
 	setupEnv()
