@@ -1,3 +1,4 @@
+//go:build wasm && !wasi
 // +build wasm,!wasi
 
 package runtime
@@ -6,18 +7,19 @@ import "unsafe"
 
 type timeUnit float64 // time in milliseconds, just like Date.now() in JavaScript
 
+// wasmNested is used to detect scheduler nesting (WASM calls into JS calls back into WASM).
+// When this happens, we need to use a reduced version of the scheduler.
+var wasmNested bool
+
 //export _start
 func _start() {
 	// These need to be initialized early so that the heap can be initialized.
 	heapStart = uintptr(unsafe.Pointer(&heapStartSymbol))
 	heapEnd = uintptr(wasm_memory_size(0) * wasmPageSize)
 
+	wasmNested = true
 	run()
-}
-
-//go:linkname syscall_runtime_envs syscall.runtime_envs
-func syscall_runtime_envs() []string {
-	return nil
+	wasmNested = false
 }
 
 var handleEvent func()
@@ -27,19 +29,6 @@ func setEventHandler(fn func()) {
 	handleEvent = fn
 }
 
-//export resume
-func resume() {
-	go func() {
-		handleEvent()
-	}()
-	scheduler()
-}
-
-//export go_scheduler
-func go_scheduler() {
-	scheduler()
-}
-
 func ticksToNanoseconds(ticks timeUnit) int64 {
 	// The JavaScript API works in float64 milliseconds, so convert to
 	// nanoseconds first before converting to a timeUnit (which is a float64),
